@@ -19,6 +19,7 @@
  */
 
 #include <config.h>
+#include <sys/types.h>
 #include <sys/stat.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -329,9 +330,11 @@ dmapd_daap_record_to_blob (DMAPRecord *record)
 static gboolean
 dmapd_daap_record_set_from_blob (DMAPRecord *_record, GByteArray *blob)
 {
+	struct stat buf;
 	gboolean fnval = FALSE;
 	DmapdDAAPRecord *record = NULL;
 	GFile *file = NULL;
+	gchar *path = NULL;
 	GFileInputStream *stream = NULL;
 	GError *error = NULL;
 	guint8 *ptr = blob->data;
@@ -443,11 +446,28 @@ dmapd_daap_record_set_from_blob (DMAPRecord *_record, GByteArray *blob)
                 goto _done;
 	}
 
-	if (! dmapd_util_hash_file (location, hash2)
-         || memcmp (hash->data, hash2, DMAP_HASH_SIZE)) {
-		g_warning ("Media file has changed since being cached\n");
-		goto _done;
-        }
+	if (FALSE /* FIXME: calculate_media_hashes */) {
+		if (! dmapd_util_hash_file (location, hash2)
+		 || memcmp (hash->data, hash2, DMAP_HASH_SIZE)) {
+			g_warning ("Media file hash has changed since being cached\n");
+			goto _done;
+		}
+	} else {
+		path = g_file_get_path(file);
+		if (NULL == path) {
+			g_warning ("Could not convert %s to local path; assuming modified\n", location);
+			goto _done;
+		}
+		if (stat (path, &buf) == -1) {
+			g_warning ("Unable to determine mtime of %s; assuming modified", path);
+			goto _done;
+		} else {
+			if (buf.st_mtime > mtime) {
+				g_warning ("Media file mtime has changed since being cached\n");
+				goto _done;
+			}
+		}
+	}
 
 	record = DMAPD_DAAP_RECORD (_record);
 
@@ -477,6 +497,10 @@ _done:
 		g_object_unref (file);
 	}
 
+	if (NULL != path) {
+		g_free (path);
+	}
+
 	if (NULL != stream) {
 		g_object_unref (stream);
 	}
@@ -598,9 +622,13 @@ DmapdDAAPRecord *dmapd_daap_record_new (const char *path, AVMetaReader *reader)
 			goto _done;
 		}
 
-		if (! dmapd_util_hash_file (location, hash_buf)) {
-			g_warning ("Unable to hash %s\n", location);
-			goto _done;
+		if (FALSE /* FIXME: calculate_media_hashes */) {
+			if (! dmapd_util_hash_file (location, hash_buf)) {
+				g_warning ("Unable to hash %s\n", location);
+				goto _done;
+			}
+		} else {
+			memset(hash_buf, 0x00, DMAP_HASH_SIZE);
 		}
 
 		g_byte_array_append (hash, hash_buf, DMAP_HASH_SIZE);
@@ -618,7 +646,7 @@ DmapdDAAPRecord *dmapd_daap_record_new (const char *path, AVMetaReader *reader)
 			             "filesize",
 				     (guint64) buf.st_size,
 				     "mtime",
-				     (guint64) buf.st_mtime,
+				      buf.st_mtime,
 				      NULL);
 		}
 
@@ -631,7 +659,8 @@ DmapdDAAPRecord *dmapd_daap_record_new (const char *path, AVMetaReader *reader)
 		                      "format",      unknown,
 		                      "mediakind",   DMAP_MEDIA_KIND_MUSIC,
 		                      "year",        1985,
-		                      "disc",        1, NULL);
+		                      "disc",        1,
+		                       NULL);
 
 		if (! av_meta_reader_read (AV_META_READER (reader), DAAP_RECORD (record), path)) {
 			g_object_unref (record);
